@@ -1,9 +1,13 @@
+import os
 from os import environ, getenv
-from typing import Literal, TypedDict
+from typing import Literal
 
 import orjson
 from starlette.config import Config, undefined
 from starlette.datastructures import CommaSeparatedStrings
+from typing_extensions import TypedDict
+
+from langgraph_api import traceblock
 
 # types
 
@@ -36,6 +40,8 @@ class HttpConfig(TypedDict, total=False):
     """Disable /store routes"""
     disable_meta: bool
     """Disable /ok, /info, /metrics, and /docs routes"""
+    disable_webhooks: bool
+    """Disable webhooks calls on run completion in all routes"""
     cors: CorsConfig | None
     """CORS configuration"""
     disable_ui: bool
@@ -149,6 +155,11 @@ STATS_INTERVAL_SECS = env("STATS_INTERVAL_SECS", cast=int, default=60)
 DATABASE_URI = env("DATABASE_URI", cast=str, default=getenv("POSTGRES_URI", undefined))
 MIGRATIONS_PATH = env("MIGRATIONS_PATH", cast=str, default="/storage/migrations")
 POSTGRES_POOL_MAX_SIZE = env("LANGGRAPH_POSTGRES_POOL_MAX_SIZE", cast=int, default=150)
+RESUMABLE_STREAM_TTL_SECONDS = env(
+    "RESUMABLE_STREAM_TTL_SECONDS",
+    cast=int,
+    default=120,  # 2 minutes
+)
 
 
 def _get_encryption_key(key_str: str | None):
@@ -165,8 +176,9 @@ LANGGRAPH_AES_KEY = env("LANGGRAPH_AES_KEY", default=None, cast=_get_encryption_
 # redis
 REDIS_URI = env("REDIS_URI", cast=str)
 REDIS_CLUSTER = env("REDIS_CLUSTER", cast=bool, default=False)
-REDIS_MAX_CONNECTIONS = env("REDIS_MAX_CONNECTIONS", cast=int, default=500)
+REDIS_MAX_CONNECTIONS = env("REDIS_MAX_CONNECTIONS", cast=int, default=2000)
 REDIS_CONNECT_TIMEOUT = env("REDIS_CONNECT_TIMEOUT", cast=float, default=10.0)
+REDIS_MAX_IDLE_TIME = env("REDIS_MAX_IDLE_TIME", cast=float, default=120.0)
 REDIS_KEY_PREFIX = env("REDIS_KEY_PREFIX", cast=str, default="")
 
 # server
@@ -239,7 +251,9 @@ BG_JOB_INTERVAL = 30  # seconds
 BG_JOB_MAX_RETRIES = 3
 BG_JOB_ISOLATED_LOOPS = env("BG_JOB_ISOLATED_LOOPS", cast=bool, default=False)
 BG_JOB_SHUTDOWN_GRACE_PERIOD_SECS = env(
-    "BG_JOB_SHUTDOWN_GRACE_PERIOD_SECS", cast=int, default=3600
+    "BG_JOB_SHUTDOWN_GRACE_PERIOD_SECS",
+    cast=int,
+    default=180,  # 3 minutes
 )
 MAX_STREAM_CHUNK_SIZE_BYTES = env(
     "MAX_STREAM_CHUNK_SIZE_BYTES", cast=int, default=1024 * 1024 * 128
@@ -353,3 +367,13 @@ API_VARIANT = env("LANGSMITH_LANGGRAPH_API_VARIANT", cast=str, default="")
 
 # UI
 UI_USE_BUNDLER = env("LANGGRAPH_UI_BUNDLER", cast=bool, default=False)
+IS_QUEUE_ENTRYPOINT = False
+
+if not os.getenv("LANGCHAIN_REVISION_ID") and (
+    ref_sha := os.getenv("LANGSMITH_LANGGRAPH_GIT_REF_SHA")
+):
+    # This is respected by the langsmith SDK env inference
+    # https://github.com/langchain-ai/langsmith-sdk/blob/1b93e4c13b8369d92db891ae3babc3e2254f0e56/python/langsmith/env/_runtime_env.py#L190
+    os.environ["LANGCHAIN_REVISION_ID"] = ref_sha
+
+traceblock.patch_requests()
