@@ -9,25 +9,24 @@ import warnings
 from collections.abc import AsyncIterator, Callable
 from contextlib import asynccontextmanager
 from itertools import filterfalse
-from random import choice
-from typing import TYPE_CHECKING, Any, NamedTuple
+from typing import TYPE_CHECKING, Any, NamedTuple, cast
 from uuid import UUID, uuid5
 
 import orjson
 import structlog
-from langchain_core.runnables.config import run_in_executor, var_child_runnable_config
 from langgraph.checkpoint.base import BaseCheckpointSaver
-from langgraph.constants import CONFIG_KEY_CHECKPOINTER, CONFIG_KEY_STORE
+from langgraph.constants import CONFIG_KEY_CHECKPOINTER
 from langgraph.graph import StateGraph
 from langgraph.pregel import Pregel
 from langgraph.store.base import BaseStore
-from langgraph.utils.config import ensure_config
 from starlette.exceptions import HTTPException
 
 from langgraph_api import asyncio as lg_asyncio
 from langgraph_api import config
+from langgraph_api.feature_flags import USE_RUNTIME_API
 from langgraph_api.js.base import BaseRemotePregel, is_js_path
 from langgraph_api.schema import Config
+from langgraph_api.utils.config import run_in_executor, var_child_runnable_config
 
 if TYPE_CHECKING:
     from langchain_core.embeddings import Embeddings
@@ -59,23 +58,32 @@ async def register_graph(
     GRAPHS[graph_id] = graph
     if callable(graph):
         FACTORY_ACCEPTS_CONFIG[graph_id] = len(inspect.signature(graph).parameters) > 0
-    async with connect() as conn:
-        graph_name = getattr(graph, "name", None) if isinstance(graph, Pregel) else None
-        assistant_name = (
-            graph_name
-            if graph_name is not None and graph_name != "LangGraph"
-            else graph_id
-        )
-        await Assistants.put(
-            conn,
-            str(uuid5(NAMESPACE_GRAPH, graph_id)),
-            graph_id=graph_id,
-            metadata={"created_by": "system"},
-            config=config or {},
-            if_exists="do_nothing",
-            name=assistant_name,
-            description=description,
-        )
+
+    from langgraph_runtime.retry import retry_db
+
+    @retry_db
+    async def register_graph_db():
+        async with connect() as conn:
+            graph_name = (
+                getattr(graph, "name", None) if isinstance(graph, Pregel) else None
+            )
+            assistant_name = (
+                graph_name
+                if graph_name is not None and graph_name != "LangGraph"
+                else graph_id
+            )
+            await Assistants.put(
+                conn,
+                str(uuid5(NAMESPACE_GRAPH, graph_id)),
+                graph_id=graph_id,
+                metadata={"created_by": "system"},
+                config=config or {},
+                if_exists="do_nothing",
+                name=assistant_name,
+                description=description,
+            )
+
+    await register_graph_db()
 
 
 def register_graph_sync(
@@ -115,18 +123,35 @@ async def get_graph(
     store: BaseStore | None = None,
 ) -> AsyncIterator[Pregel]:
     """Return the runnable."""
+    from langgraph_api.utils import config as lg_config
+
     assert_graph_exists(graph_id)
     value = GRAPHS[graph_id]
-    token = None
     if graph_id in FACTORY_ACCEPTS_CONFIG:
-        config = ensure_config(config)
-        if store is not None and not config["configurable"].get(CONFIG_KEY_STORE):
-            config["configurable"][CONFIG_KEY_STORE] = store
+        config = lg_config.ensure_config(config)
+
+        if store is not None:
+            if USE_RUNTIME_API:
+                from langgraph._internal._constants import CONFIG_KEY_RUNTIME
+                from langgraph.runtime import Runtime
+
+                if (
+                    (runtime := config["configurable"].get(CONFIG_KEY_RUNTIME))
+                    is not None
+                ) and runtime.store is None:
+                    patched_runtime = cast(Runtime, runtime).override(store=store)
+                    config["configurable"][CONFIG_KEY_RUNTIME] = patched_runtime
+            else:
+                from langgraph.constants import CONFIG_KEY_STORE
+
+                if not config["configurable"].get(CONFIG_KEY_STORE):
+                    config["configurable"][CONFIG_KEY_STORE] = store
+
         if checkpointer is not None and not config["configurable"].get(
             CONFIG_KEY_CHECKPOINTER
         ):
             config["configurable"][CONFIG_KEY_CHECKPOINTER] = checkpointer
-        token = var_child_runnable_config.set(config)
+        var_child_runnable_config.set(config)
         value = value(config) if FACTORY_ACCEPTS_CONFIG[graph_id] else value()
     try:
         async with _generate_graph(value) as graph_obj:
@@ -147,8 +172,7 @@ async def get_graph(
                 update["config"] = config
             yield graph_obj.copy(update=update)
     finally:
-        if token is not None:
-            var_child_runnable_config.reset(token)
+        var_child_runnable_config.set(None)
 
 
 def graph_exists(graph_id: str) -> bool:
@@ -159,7 +183,10 @@ def graph_exists(graph_id: str) -> bool:
 def assert_graph_exists(graph_id: str) -> None:
     """Assert that a graph exists."""
     if not graph_exists(graph_id):
-        raise HTTPException(status_code=404, detail=f"Graph '{graph_id}' not found")
+        raise HTTPException(
+            status_code=404,
+            detail=f"Graph '{graph_id}' not found. Expected one of: {sorted(GRAPHS.keys())}",
+        )
 
 
 def get_assistant_id(assistant_id: str) -> str:
@@ -407,7 +434,12 @@ def _graph_from_spec(spec: GraphSpec) -> GraphValue:
         module = importlib.import_module(spec.module)
     elif spec.path:
         try:
-            modname = "".join(choice("abcdefghijklmnopqrstuvwxyz") for _ in range(24))
+            modname = (
+                spec.path.replace("/", "__")
+                .replace(".py", "")
+                .replace(" ", "_")
+                .lstrip(".")
+            )
             modspec = importlib.util.spec_from_file_location(modname, spec.path)
             if modspec is None:
                 raise ValueError(f"Could not find python file for graph: {spec}")
@@ -560,8 +592,8 @@ def resolve_embeddings(index_config: dict) -> "Embeddings":
         try:
             if "/" in module_name:
                 # Load from file path
-                modname = "".join(
-                    choice("abcdefghijklmnopqrstuvwxyz") for _ in range(24)
+                modname = (
+                    module_name.replace("/", "__").replace(".py", "").replace(" ", "_")
                 )
                 modspec = importlib.util.spec_from_file_location(modname, module_name)
                 if modspec is None:
