@@ -1,6 +1,8 @@
 import asyncio
+import json
 import logging
 import os
+import re
 import shutil
 import ssl
 from collections import deque
@@ -22,9 +24,8 @@ from langchain_core.runnables.schema import (
     StreamEvent,
 )
 from langgraph.checkpoint.serde.base import SerializerProtocol
-from langgraph.pregel.types import PregelTask, StateSnapshot
 from langgraph.store.base import GetOp, Item, ListNamespacesOp, PutOp, SearchOp
-from langgraph.types import Command, Interrupt, Send
+from langgraph.types import Command, Interrupt, PregelTask, Send, StateSnapshot
 from langgraph_sdk import Auth
 from pydantic import BaseModel
 from starlette import types
@@ -79,6 +80,8 @@ def _snapshot_defaults():
 def default_command(obj):
     if isinstance(obj, Send):
         return {"node": obj.node, "args": obj.arg}
+    if isinstance(obj, ProxyUser):
+        return obj.dict()
     raise TypeError
 
 
@@ -451,6 +454,26 @@ async def run_js_http_process(paths_str: str, http_config: dict, watch: bool = F
                 attempt += 1
 
 
+_BAD_SURROGATE_RE = re.compile(r"\\u[dD][89a-fA-F][0-9a-fA-F]{2}")
+_BAD_ESCAPE_RE = re.compile(r"\\(?![\"\\/bfnrtu])")
+
+
+def _safe_json_loads(data: bytes):
+    """Attempt *orjson.loads* first; if it fails, repair common escape issues.
+
+    For a time, we had a bug in our surrogate cleanup in serde.py, which
+    allowed sequences containing a stray backslash to be stored which would
+    then fail upon loading. This function attempts to repair those sequences.
+    """
+    try:
+        return orjson.loads(data)
+    except orjson.JSONDecodeError:
+        txt = data.decode("utf-8", "replace")
+        txt = _BAD_ESCAPE_RE.sub(r"\\\\", txt)
+        txt = _BAD_SURROGATE_RE.sub("", txt)
+        return json.loads(txt)
+
+
 class PassthroughSerialiser(SerializerProtocol):
     def dumps(self, obj: Any) -> bytes:
         return json_dumpb(obj)
@@ -459,13 +482,13 @@ class PassthroughSerialiser(SerializerProtocol):
         return "json", json_dumpb(obj)
 
     def loads(self, data: bytes) -> Any:
-        return orjson.loads(data)
+        return _safe_json_loads(data)
 
     def loads_typed(self, data: tuple[str, bytes]) -> Any:
         type, payload = data
         if type != "json":
             raise ValueError(f"Unsupported type {type}")
-        return orjson.loads(payload)
+        return _safe_json_loads(payload)
 
 
 def _get_passthrough_checkpointer():
@@ -858,6 +881,16 @@ async def handle_js_auth_event(
     ctx: Auth.types.AuthContext | None,
     value: dict,
 ) -> Auth.types.FilterType | None:
+    if hasattr(ctx.user, "dict") and callable(ctx.user.dict):
+        user = ctx.user.dict()
+    else:
+        user = {
+            "is_authenticated": ctx.user.is_authenticated,
+            "display_name": ctx.user.display_name,
+            "identity": ctx.user.identity,
+            "permissions": ctx.user.permissions,
+        }
+
     res = await _client.post(
         "/auth/authorize",
         headers={"Content-Type": "application/json"},
@@ -868,7 +901,7 @@ async def handle_js_auth_event(
                 "value": value,
                 "context": (
                     {
-                        "user": cast(DotDict, ctx.user).dict(),
+                        "user": user,
                         "scopes": ctx.permissions,
                     }
                     if ctx
