@@ -5,7 +5,7 @@ import structlog
 from starlette.requests import ClientDisconnect
 from starlette.types import Message, Receive, Scope, Send
 
-from langgraph_api.logging import LOG_JSON
+from langgraph_api.http_metrics import HTTP_METRICS_COLLECTOR
 
 asgi = structlog.stdlib.get_logger("asgi")
 
@@ -28,7 +28,7 @@ class AccessLoggerMiddleware:
             self.debug_enabled = False
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
-        if scope["type"] != "http" or (LOG_JSON and scope.get("path") in PATHS_IGNORE):
+        if scope["type"] != "http" or scope.get("path") in PATHS_IGNORE:
             return await self.app(scope, receive, send)  # pragma: no cover
 
         loop = asyncio.get_event_loop()
@@ -66,13 +66,22 @@ class AccessLoggerMiddleware:
         finally:
             info["end_time"] = loop.time()
             latency = int((info["end_time"] - info["start_time"]) * 1_000)
+
+            status = info["response"].get("status")
+            method = scope.get("method")
+            path = scope.get("path")
+            route = scope.get("route")
+
+            if method and route and status:
+                HTTP_METRICS_COLLECTOR.record_request(method, route, status, latency)
+
             self.logger.info(
-                f"{scope.get('method')} {scope.get('path')} {info['response'].get('status')} {latency}ms",
-                method=scope.get("method"),
-                path=scope.get("path"),
-                status=info["response"].get("status"),
+                f"{method} {path} {status} {latency}ms",
+                method=method,
+                path=path,
+                status=status,
                 latency_ms=latency,
-                route=scope.get("route"),
+                route=route,
                 path_params=scope.get("path_params"),
                 query_string=scope.get("query_string").decode(),
                 proto=scope.get("http_version"),
