@@ -1,16 +1,18 @@
 import asyncio
 import functools
 import re
+import time
 import urllib.parse
 import uuid
 from collections.abc import Mapping, Sequence
-from typing import Any, NamedTuple, TypedDict
+from typing import Any, NamedTuple
 from uuid import UUID
 
 import orjson
 from langgraph.checkpoint.base.id import uuid6
 from starlette.authentication import BaseUser
 from starlette.exceptions import HTTPException
+from typing_extensions import TypedDict
 
 from langgraph_api.graph import GRAPHS, get_assistant_id
 from langgraph_api.schema import (
@@ -28,6 +30,13 @@ from langgraph_api.utils import AsyncConnectionProto, get_auth_ctx
 from langgraph_runtime.ops import Runs, logger
 
 
+class LangSmithTracer(TypedDict, total=False):
+    """Configuration for LangSmith tracing."""
+
+    example_id: str | None
+    project_name: str | None
+
+
 class RunCreateDict(TypedDict):
     """Payload for creating a run."""
 
@@ -86,6 +95,8 @@ class RunCreateDict(TypedDict):
     """Start the run after this many seconds. Defaults to 0."""
     if_not_exists: IfNotExists
     """Create the thread if it doesn't exist. If False, reply with 404."""
+    langsmith_tracer: LangSmithTracer | None
+    """Configuration for additional tracing with LangSmith."""
 
 
 def ensure_ids(
@@ -193,11 +204,11 @@ def get_header_patterns() -> tuple[
     configurable = config.HTTP_CONFIG.get("configurable_headers")
     if not configurable:
         return None, None
-    header_includes = configurable.get("includes") or []
+    header_includes = configurable.get("includes") or configurable.get("include") or []
     include_patterns = []
     for include in header_includes:
         include_patterns.append(translate_pattern(include))
-    header_excludes = configurable.get("excludes") or []
+    header_excludes = configurable.get("excludes") or configurable.get("exclude") or []
     exclude_patterns = []
     for exclude in header_excludes:
         exclude_patterns.append(translate_pattern(exclude))
@@ -249,6 +260,7 @@ async def create_valid_run(
     headers: Mapping[str, str],
     barrier: asyncio.Barrier | None = None,
     run_id: UUID | None = None,
+    request_start_time: float | None = None,
 ) -> Run:
     request_id = headers.get("x-request-id")  # Will be null in the crons scheduler.
     (
@@ -293,6 +305,15 @@ async def create_valid_run(
         user_id = None
     if not configurable.get("langgraph_request_id"):
         configurable["langgraph_request_id"] = request_id
+    if ls_tracing := payload.get("langsmith_tracer"):
+        configurable["__langsmith_project__"] = ls_tracing.get("project_name")
+        configurable["__langsmith_example_id__"] = ls_tracing.get("example_id")
+    if request_start_time:
+        configurable["__request_start_time_ms__"] = request_start_time
+    after_seconds = payload.get("after_seconds", 0)
+    configurable["__after_seconds__"] = after_seconds
+    put_time_start = time.time()
+    if_not_exists = payload.get("if_not_exists", "reject")
     run_coro = Runs.put(
         conn,
         assistant_id,
@@ -317,8 +338,8 @@ async def create_valid_run(
         run_id=run_id,
         multitask_strategy=multitask_strategy,
         prevent_insert_if_inflight=prevent_insert_if_inflight,
-        after_seconds=payload.get("after_seconds", 0),
-        if_not_exists=payload.get("if_not_exists", "reject"),
+        after_seconds=after_seconds,
+        if_not_exists=if_not_exists,
     )
     run_ = await run_coro
 
@@ -344,8 +365,14 @@ async def create_valid_run(
             multitask_strategy=multitask_strategy,
             stream_mode=stream_mode,
             temporary=temporary,
-            after_seconds=payload.get("after_seconds", 0),
-            if_not_exists=payload.get("if_not_exists", "reject"),
+            after_seconds=after_seconds,
+            if_not_exists=if_not_exists,
+            run_create_ms=(
+                int(time.time() * 1_000) - request_start_time
+                if request_start_time
+                else None
+            ),
+            run_put_ms=int((time.time() - put_time_start) * 1_000),
         )
         # inserted, proceed
         if multitask_strategy in ("interrupt", "rollback") and inflight_runs:
